#include "StdAfx.h"
#include "Application.h"

CApplication::CApplication(void)
{
	CurrentScene = NULL;
}

CApplication::~CApplication(void)
{
	delete CurrentScene;
}

void CApplication::Start(void) {
	InitWindow();
	InitGL();

	CurrentScene = new CScene();

	CurrentScene->Init();
	GameLoop();

	Cleanup();
}

void CApplication::InitWindow(void) {

	glfwSetErrorCallback(HandleGLFWError);

	if (!glfwInit()) {
		printf("ERROR: Cannot initialize GLFW!\n");
		exit(EXIT_FAILURE);
	}
	
	glfwWindowHint(GLFW_SAMPLES, 16);
	glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, OPENGL_MAJOR);
	glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, OPENGL_MINOR);

	window = glfwCreateWindow(640, 480, "PGK Lab 1", NULL, NULL);
	
	if (!window) {
		printf("ERROR: Cannot create window!\n");
		glfwTerminate();
		exit(EXIT_FAILURE);
	}

	glfwMakeContextCurrent(window);

	glfwSetTime(0.0);
}

void CApplication::InitGL(void) {
	glClearColor(0, 0, 0, 0);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glEnable(GL_NORMALIZE);
	glewInit();
}

void CApplication::ProcessEvents(void) {
	glfwPollEvents();
	CInput::GetInstance()->HandleInput();
}

void CApplication::Cleanup(void) {
	glfwTerminate();
}

double CApplication::GetTime(void) {
	return glfwGetTime();
}


void CApplication::Redraw() {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	CurrentScene->Redraw();

	glfwSwapBuffers(window);
}


void CApplication::Update() {
	ProcessEvents();
	CurrentScene->Update();
}


void CApplication::GameLoop() {
	
	double currentTime = GetTime();
	double nextUpdateTime = GetTime();

	int MaxFrameskip = 10;

	while (!CInput::GetInstance()->UserWantsToExit()) {
		currentTime = GetTime();

		int substeps = 0;
		while ((currentTime >= nextUpdateTime) && (substeps < MaxFrameskip)) {
			Update();
			++substeps;
			nextUpdateTime += UPDATE_INTERVAL;
		}

		Redraw();
	}

}

void CApplication::HandleGLFWError(int error, const char* description) {
	printf("GLFW Error (%d): %s\n", error, description);
}

