#include "StdAfx.h"
#include "Camera.h"

CCamera::CCamera(void)
{
	Position = glm::vec3(0, 0, 0);
	Direction = glm::vec3(0, 0, -1);
	Up = glm::vec3(0, 1, 0);
	MovementSpeed = glm::vec2(0, 0);
	RotationSpeed = glm::vec2(0, 0);
}

CCamera::~CCamera(void)
{
}

void CCamera::SetViewTransform(void)
{
	glm::vec3 pos = Position;
	glm::vec3 lookAt = Position + Direction;
	glm::vec3 up = Up;
	gluLookAt(
		pos.x, pos.y, pos.z,
		lookAt.x, lookAt.y, lookAt.z,
		up.x, up.y, up.z
	);
}

void CCamera::SetProjectionTransform(void)
{	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	int w, h;
	glfwGetFramebufferSize(CApplication::GetInstance()->window, &w, &h);
	gluPerspective(60.0, (double) w / h, .1, 1000.0);
	glViewport(0, 0, w, h);
}

void CCamera::Update(void)
{
	HandleUserInput();

	float T = acos(Direction.y);
	float G = atan2(Direction.z, Direction.x);
	T -= RotationSpeed.x * .035f;
	G += RotationSpeed.y * .035f;

	T = __max(PI / 32, __min(PI - PI / 32, T)); // Vertical rotation limit
	
	Direction.x = sin(T) * cos(G);
	Direction.y = cos(T);
	Direction.z = sin(T) * sin(G);

	glm::vec3 movementOffset = Direction;
	movementOffset *= MovementSpeed.x * .2;
	Position += movementOffset;
	
	glm::vec3 strafeOffset(-Direction.z, 0, Direction.x);
	strafeOffset *= MovementSpeed.y * .2;
	Position += strafeOffset;

	MovementSpeed /= 1.05;
	RotationSpeed /= 1.3;
}

void CCamera::HandleUserInput(void)
{
	bool run = false;
	if (glfwGetKey(CApplication::GetInstance()->window, GLFW_KEY_LEFT_SHIFT)) {
		run = true;
	}
	if (glfwGetKey(CApplication::GetInstance()->window, GLFW_KEY_W)) {
		MovementSpeed.x = run ? 3.0 : 1.0;
	}
	if (glfwGetKey(CApplication::GetInstance()->window, GLFW_KEY_S)) {
		MovementSpeed.x = run ? -3.0 : -1.0;
	}
	if (glfwGetKey(CApplication::GetInstance()->window, GLFW_KEY_D)) {
		MovementSpeed.y = run ? 3.0 : 1.0;
	}
	if (glfwGetKey(CApplication::GetInstance()->window, GLFW_KEY_A)) {
		MovementSpeed.y = run ? -3.0 : -1.0;
	}
	if (glfwGetKey(CApplication::GetInstance()->window, GLFW_KEY_DOWN)) {
		RotationSpeed.x = 1.0;
	}
	if (glfwGetKey(CApplication::GetInstance()->window, GLFW_KEY_UP)) {
		RotationSpeed.x = -1.0;
	}
	if (glfwGetKey(CApplication::GetInstance()->window, GLFW_KEY_RIGHT)) {
		RotationSpeed.y = 1.0;
	}
	if (glfwGetKey(CApplication::GetInstance()->window, GLFW_KEY_LEFT)) {
		RotationSpeed.y = -1.0;
	}
}
