#include "StdAfx.h"
#include "Scene.h"

CScene::CScene(void)
{
	camera = new CCamera();
	camera->Position.y = 11;
	sunAngle = 0.0f;
}

CScene::~CScene(void)
{
	delete camera;
}

void CScene::Init(void) {
	CreateVBO();
}

void CScene::Redraw(void) {
	SetLighting();
	camera->SetProjectionTransform();
	camera->SetViewTransform();
	DrawTerrain();
}

void CScene::Update(void) {
	sunAngle += .003f;
	camera->Update();
}

void CScene::SetLighting(void) {
	glEnable(GL_LIGHT0);
	GLfloat l1Ambient[] = {0.3f, 0.3f, 0.3f};
	GLfloat l1Diffuse[] = {0.75f, 0.85f, 1.0f};
	GLfloat l1Specular[] = {0.0f, 0.0f, 0.0f};
	glLightfv(GL_LIGHT0, GL_AMBIENT, l1Ambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, l1Diffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, l1Specular);
	GLfloat l1Pos[] = {sin(sunAngle), 1.0f, cos(sunAngle), 0.0f};
	glLightfv(GL_LIGHT0, GL_POSITION, l1Pos);
}

void CScene::DrawTerrain(void) {
	GLfloat mAmbient[] = {.1f, 0.1f, 0.1f};
	GLfloat mDiffuse[] = {1.0f, 1.0f, 1.0f};
	GLfloat mSpecular[] = {1.0f, 1.0f, 1.0f};
	glMaterialfv(GL_FRONT, GL_AMBIENT, mAmbient);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, mDiffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR, mSpecular);

	glBindBuffer(GL_ARRAY_BUFFER, vboId);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
}


#define VBO_STRIDE 6

void CScene::CreateVBO(void) {
	
	glGenBuffers(1, &vboId);
	glBindBuffer(GL_ARRAY_BUFFER, vboId);

	GLfloat * buf = new GLfloat[VBO_STRIDE * 4];
	
	buf[0 * VBO_STRIDE + 0] = -100.0f;
	buf[0 * VBO_STRIDE + 1] = 0.0f;
	buf[0 * VBO_STRIDE + 2] = 100.0f;
	buf[0 * VBO_STRIDE + 3] = 0.0f;
	buf[0 * VBO_STRIDE + 4] = 1.0f;
	buf[0 * VBO_STRIDE + 5] = 0.0f;

	buf[1 * VBO_STRIDE + 0] = 100.0f;
	buf[1 * VBO_STRIDE + 1] = 0.0f;
	buf[1 * VBO_STRIDE + 2] = 100.0f;
	buf[1 * VBO_STRIDE + 3] = 0.0f;
	buf[1 * VBO_STRIDE + 4] = 1.0f;
	buf[1 * VBO_STRIDE + 5] = 0.0f;

	buf[2 * VBO_STRIDE + 0] = -100.0f;
	buf[2 * VBO_STRIDE + 1] = 0.0f;
	buf[2 * VBO_STRIDE + 2] = -100.0f;
	buf[2 * VBO_STRIDE + 3] = 0.0f;
	buf[2 * VBO_STRIDE + 4] = 1.0f;
	buf[2 * VBO_STRIDE + 5] = 0.0f;

	buf[3 * VBO_STRIDE + 0] = 100.0f;
	buf[3 * VBO_STRIDE + 1] = 0.0f;
	buf[3 * VBO_STRIDE + 2] = -100.0f;
	buf[3 * VBO_STRIDE + 3] = 0.0f;
	buf[3 * VBO_STRIDE + 4] = 1.0f;
	buf[3 * VBO_STRIDE + 5] = 0.0f;
	
	glBufferData(GL_ARRAY_BUFFER, VBO_STRIDE * 4 * sizeof(GLfloat), buf, GL_STATIC_DRAW);

	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_NORMAL_ARRAY);
	glVertexPointer(3, GL_FLOAT, VBO_STRIDE * sizeof(GLfloat), 0);
	glNormalPointer(GL_FLOAT, VBO_STRIDE * sizeof(GLfloat), (const GLvoid *) (3 * sizeof(GLfloat)));
	
	delete buf;
}
